#include <stdio.h>

#include "bs_cairo.h"
#include "bs_mem.h"
#include "bs_assert.h"

#define BS_CAIRO_START_X 100
#define BS_CAIRO_START_Y 100

/* X11 */ 

bs_cairo_x11_t* 
bs_cairo_x11_create(bs_int32_t width, 
                    bs_int32_t height)
{
  bs_cairo_x11_t *pCairoX11; 
  Window rootWindow; 
  Visual *pVisual; 

  BS_MEM_ALLOC_TYPE(pCairoX11, bs_cairo_x11_t, 1); 

  pCairoX11->width = width; 
  pCairoX11->height = height; 
  
  pCairoX11->pDisplay = XOpenDisplay(0); 
  if(pCairoX11->pDisplay == NULL)
  {
    fprintf(stderr, "Cannor open X11 display\n"); 
    exit(1); 
  }    
  
  rootWindow = DefaultRootWindow(pCairoX11->pDisplay);
  pCairoX11->screen = DefaultScreen(pCairoX11->pDisplay);

  pCairoX11->window = XCreateSimpleWindow(pCairoX11->pDisplay, rootWindow, 
    BS_CAIRO_START_X, BS_CAIRO_START_Y, width, height, 0, 
    BlackPixel(pCairoX11->pDisplay, pCairoX11->screen), 
    WhitePixel(pCairoX11->pDisplay, pCairoX11->screen));

  pCairoX11->quitCode = XKeysymToKeycode(pCairoX11->pDisplay, XStringToKeysym("Q"));

  XSelectInput(pCairoX11->pDisplay, pCairoX11->window,
    KeyPressMask | StructureNotifyMask | ExposureMask);

  XMapWindow(pCairoX11->pDisplay, pCairoX11->window);
  
  pVisual = DefaultVisual(pCairoX11->pDisplay, DefaultScreen(pCairoX11->pDisplay));
  
  XClearWindow(pCairoX11->pDisplay, pCairoX11->window);

  pCairoX11->pCairoSurface = cairo_xlib_surface_create(pCairoX11->pDisplay, 
    pCairoX11->window, pVisual, width, height);
  
  pCairoX11->pCairo = cairo_create(pCairoX11->pCairoSurface);

  return pCairoX11; 
}

void 
bs_cairo_x11_destory(bs_cairo_x11_t *pCairoX11)
{
  BS_ASSERT(pCairoX11 != NULL); 
  BS_ASSERT(pCairoX11->pDisplay != NULL); 
  BS_ASSERT(pCairoX11->pCairoSurface != NULL); 
  BS_ASSERT(pCairoX11->pCairo != NULL); 
  
  cairo_destroy(pCairoX11->pCairo);
  cairo_surface_destroy(pCairoX11->pCairoSurface);

  XDestroyWindow(pCairoX11->pDisplay, pCairoX11->window);
  XCloseDisplay(pCairoX11->pDisplay);

  BS_MEM_FREE(pCairoX11); 
}

cairo_t* 
bs_cairo_x11_get_cairo(bs_cairo_x11_t *pCairoX11)
{
  BS_ASSERT(pCairoX11 != NULL);
  return pCairoX11->pCairo;   
}

/* GTK */ 

#if ! GTK_CHECK_VERSION(2,7,0)
/* copied from gtk+/gdk/gdkcairo.c and gtk+/gdk/x11/gdkdrawable-x11.c
 * gdk_cairo_create() which is available in 2.7.0 and later.
 */
static cairo_t *
gdk_cairo_create (GdkDrawable *drawable)
{
  int width, height;
  cairo_t *cr = NULL;
  cairo_surface_t *surface = NULL;
  GdkVisual *visual = gdk_drawable_get_visual (drawable);
  GdkDrawable *dr;
  gint x_off, y_off;

  gdk_drawable_get_size (drawable, &width, &height);
  if (visual) 
  {
    gdk_window_get_internal_paint_info(drawable, &dr, &x_off, &y_off);
    surface = cairo_xlib_surface_create (GDK_DRAWABLE_XDISPLAY (drawable),
      GDK_DRAWABLE_XID (dr), GDK_VISUAL_XVISUAL (visual), width, height);
  }
  else if (gdk_drawable_get_depth (drawable) == 1)
  {
    surface = cairo_xlib_surface_create_for_bitmap 
      (GDK_PIXMAP_XDISPLAY (drawable), GDK_PIXMAP_XID (drawable), width, height);
  } 
  else 
  {
    g_warning ("Using Cairo rendering requires the drawable argument to\n"
       "have a specified colormap. All windows have a colormap,\n"
       "however, pixmaps only have colormap by default if they\n"
       "were created with a non-NULL window argument. Otherwise\n"
       "a colormap must be set on them with "
       "gdk_drawable_set_colormap");
    return NULL;
  }
  
  if (surface != NULL) 
  {
    cr = cairo_create (surface);
    cairo_surface_destroy (surface);
  }
  return cr;
}
#endif

static gboolean
bs_cairo_gtk_handle_expose(GtkWidget *widget,
                           GdkEventExpose *event,
                           gpointer data)
{
  bs_cairo_gtk_t *pCairoGtk; 
  
  pCairoGtk = (bs_cairo_gtk_t*) data; 
  pCairoGtk->handleExposeCb(pCairoGtk->pUserData); 
  
  return FALSE;
}

bs_cairo_gtk_t*
bs_cairo_gtk_create(bs_int32_t width, 
                    bs_int32_t height, 
                    bs_cairo_gtk_handle_expose_t handleExposeCb, 
                    void *pUserData)
{
  bs_cairo_gtk_t *pCairoGtk; 
  
  BS_MEM_ALLOC_TYPE(pCairoGtk, bs_cairo_gtk_t, 1); 
  
  pCairoGtk->width = width; 
  pCairoGtk->height = height; 
  pCairoGtk->handleExposeCb = handleExposeCb; 
  pCairoGtk->pUserData = pUserData; 
  
  pCairoGtk->pWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  gtk_window_set_default_size(GTK_WINDOW(pCairoGtk->pWindow), 
    pCairoGtk->width, pCairoGtk->height);
  gtk_window_set_title(GTK_WINDOW(pCairoGtk->pWindow), "Blob Sallad");

  g_signal_connect(pCairoGtk->pWindow, "destroy-event",
    G_CALLBACK(gtk_main_quit), NULL);

  pCairoGtk->pDrawingArea = gtk_drawing_area_new ();
  gtk_container_add(GTK_CONTAINER(pCairoGtk->pWindow), 
    pCairoGtk->pDrawingArea);

  g_signal_connect(pCairoGtk->pDrawingArea, "expose-event",
    G_CALLBACK(bs_cairo_gtk_handle_expose), pCairoGtk);

  pCairoGtk->pCairo = NULL;
  
  gtk_widget_show_all(pCairoGtk->pWindow);
  
  return pCairoGtk;
}

void 
bs_cairo_gtk_destroy(bs_cairo_gtk_t *pCairoGtk)
{
  
}

cairo_t* 
bs_cairo_gtk_get_cairo(bs_cairo_gtk_t *pCairoGtk)
{
  cairo_t *pCairo; 
  
  BS_ASSERT(pCairoGtk != NULL);
  pCairo = gdk_cairo_create(pCairoGtk->pDrawingArea->window);
  return pCairo;   
}

/* SDL */

bs_cairo_sdl_t*
bs_cairo_sdl_create(bs_int32_t width, 
                    bs_int32_t height)
{
  bs_cairo_sdl_t *pCairoSdl; 
  cairo_surface_t *pCairoSurface; 

  BS_MEM_ALLOC_TYPE(pCairoSdl, bs_cairo_sdl_t, 1); 

  pCairoSdl->width = width; 
  pCairoSdl->height = height;

  if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0)
  {
    fprintf(stderr, "SDL_Init failed: %s\n", SDL_GetError());
    exit(1);     
  }
  
  atexit(SDL_Quit); 
  pCairoSdl->pScreen = SDL_SetVideoMode(width, height, 32, SDL_SWSURFACE); 
  if(pCairoSdl->pScreen == NULL)
  {
    fprintf(stderr, "SDL_SetVideoMode failed: %s\n", SDL_GetError()); 
    exit(1); 
  }

  pCairoSurface = cairo_image_surface_create_for_data(pCairoSdl->pScreen->pixels, 
    CAIRO_FORMAT_ARGB32, pCairoSdl->width, pCairoSdl->height, pCairoSdl->width * 4); 
  pCairoSdl->pCairo = cairo_create(pCairoSurface);
  cairo_surface_destroy(pCairoSurface);
  
  return pCairoSdl; 
}

void 
bs_cairo_sdl_destroy(bs_cairo_sdl_t *pCairoSdl)
{
  
}

void 
bs_cairo_sdl_lock_surface(bs_cairo_sdl_t *pCairoSdl)
{
  if (SDL_MUSTLOCK(pCairoSdl->pScreen))
  {
    if (SDL_LockSurface(pCairoSdl->pScreen) < 0)
    {
      return;
    }
  }
}

void 
bs_cairo_sdl_unlock_surface(bs_cairo_sdl_t *pCairoSdl)
{
  if (SDL_MUSTLOCK(pCairoSdl->pScreen))
  {
    SDL_UnlockSurface(pCairoSdl->pScreen);
  }
}
