#ifndef __BS_CAIRO_H__
#define __BS_CAIRO_H__

#include "bs_types.h"

#include <cairo.h>
#include <cairo-xlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <SDL.h>

/* X11 */ 

typedef struct bs_cairo_x11_st
{
  bs_int32_t width, height; 
  Display *pDisplay;
  int screen;
  Window window;
  GC gc;
  KeyCode quitCode;
  cairo_surface_t *pCairoSurface;
  cairo_t *pCairo;   
} bs_cairo_x11_t; 

bs_cairo_x11_t* 
bs_cairo_x11_create(bs_int32_t width, 
                    bs_int32_t height); 

void 
bs_cairo_x11_destory(bs_cairo_x11_t *pCairoX11); 

cairo_t* 
bs_cairo_x11_get_cairo(bs_cairo_x11_t *pCairoX11); 

/* GTK */ 

typedef void (*bs_cairo_gtk_handle_expose_t)(void*);

typedef struct bs_cairo_gtk_st
{
  bs_int32_t width, height; 
  GtkWidget *pWindow;
  GtkWidget *pDrawingArea;
  cairo_t *pCairo;
  bs_cairo_gtk_handle_expose_t handleExposeCb; 
  void *pUserData; 
} bs_cairo_gtk_t; 

bs_cairo_gtk_t*
bs_cairo_gtk_create(bs_int32_t width, 
                    bs_int32_t height, 
                    bs_cairo_gtk_handle_expose_t handleExposeCb, 
                    void *pUserData); 

void 
bs_cairo_gtk_destroy(bs_cairo_gtk_t *pCairoGtk); 

cairo_t* 
bs_cairo_gtk_get_cairo(bs_cairo_gtk_t *pCairoGtk); 

/* SDL */

typedef struct bs_cairo_sdl_st
{
  bs_int32_t width, height; 
  SDL_Surface *pScreen; 
  cairo_t *pCairo;
} bs_cairo_sdl_t; 

bs_cairo_sdl_t*
bs_cairo_sdl_create(bs_int32_t width, 
                    bs_int32_t height);

void 
bs_cairo_sdl_destroy(bs_cairo_sdl_t *pCairoSdl); 

#define bs_cairo_sdl_get_cairo(CAIROSDL) \
  (CAIROSDL)->pCairo

void 
bs_cairo_sdl_lock_surface(bs_cairo_sdl_t *pCairoSdl); 

void 
bs_cairo_sdl_unlock_surface(bs_cairo_sdl_t *pCairoSdl); 

#endif
