#include "bs_env.h"
#include "bs_mem.h"
#include "bs_array.h"

bs_env_t*
bs_env_create(bs_float_t x, 
              bs_float_t y, 
              bs_float_t w, 
              bs_float_t h)
{
  bs_env_t *pEnv; 

  BS_MEM_ALLOC_TYPE(pEnv, bs_env_t, 1); 
  
  pEnv->left = x; 
  pEnv->right = x + w; 
  pEnv->top = y; 
  pEnv->bottom = y + h; 
  
  return pEnv; 
}

void 
bs_env_destroy(bs_env_t *pEnv)
{
  BS_ASSERT(pEnv != NULL);
  BS_MEM_FREE(pEnv); 
}

bs_bool_t 
bs_env_collision(bs_env_t *pEnv, 
                 bs_vector_t *pCurPos, 
                 bs_vector_t *pPrevPos)
{
  bs_bool_t collision = FALSE; 
  bs_float_t x, y; 

  x = bs_vector_get_x(pCurPos); 
  y = bs_vector_get_y(pCurPos); 

  if(x < pEnv->left)
  {
    bs_vector_set_x(pCurPos, pEnv->left); 
    collision = TRUE;     
  }    
  else if(x > pEnv->right)
  {
    bs_vector_set_x(pCurPos, pEnv->right); 
    collision = TRUE;     
  }
  if(y < pEnv->top)
  {
    bs_vector_set_y(pCurPos, pEnv->top); 
    collision = TRUE;     
  }    
  else if(y > pEnv->bottom)
  {
    bs_vector_set_y(pCurPos, pEnv->bottom); 
    collision = TRUE;     
  }
  
  return collision; 
}
