#include "bs_joint.h"
#include "bs_mem.h"
#include "bs_assert.h"

bs_joint_t* 
bs_joint_create(bs_pointmass_t *pPointmassA, 
                bs_pointmass_t *pPointmassB, 
                bs_float_t shortConst, 
                bs_float_t longConst)
{
  bs_joint_t *pJoint; 
  bs_vector_t delta; 
  bs_float_t dotProd; 

  BS_MEM_ALLOC_TYPE(pJoint, bs_joint_t, 1); 
  
  pJoint->pAPos = bs_pointmass_get_pos(pPointmassA);   
  pJoint->pBPos = bs_pointmass_get_pos(pPointmassB); 

  bs_vector_set(&delta, pJoint->pBPos); 
  bs_vector_sub(&delta, pJoint->pAPos); 
  
  dotProd = bs_vector_dot_prod(&delta, &delta); 

  pJoint->shortConstSquared = dotProd * shortConst * shortConst; 
  pJoint->longConstSquared = dotProd * longConst * longConst; 
  
  return pJoint; 
}

void 
bs_joint_destroy(bs_joint_t *pJoint)
{
  BS_ASSERT(pJoint != NULL);
  BS_MEM_FREE(pJoint);   
}
  
void 
bs_joint_set_consts(bs_joint_t *pJoint, 
                    bs_float_t shortConst, 
                    bs_float_t longConst)
{
  BS_ASSERT(pJoint != NULL); 
  pJoint->shortConstSquared = shortConst * shortConst; 
  pJoint->longConstSquared = longConst * longConst;   
}

void 
bs_joint_scale(bs_joint_t *pJoint, 
               bs_float_t scaleFactor)
{
  bs_float_t scaleFactorSquared; 
  
  BS_ASSERT(pJoint != NULL); 

  scaleFactorSquared = scaleFactor * scaleFactor; 
  pJoint->shortConstSquared *= scaleFactorSquared; 
  pJoint->longConstSquared *= scaleFactorSquared;    
}

void 
bs_joint_sc(bs_joint_t *pJoint)
{
  bs_float_t dotProd, scaleFactor; 
  bs_vector_t delta; 

  BS_ASSERT(pJoint != NULL); 
  
  BS_VECTOR_SET(&delta, pJoint->pBPos); 
  BS_VECTOR_SUB(&delta, pJoint->pAPos); 
  
  dotProd = BS_VECTOR_DOT_PROD(&delta, &delta); 
  
  if(pJoint->shortConstSquared != 0.0 && 
     dotProd < pJoint->shortConstSquared)
  {
    scaleFactor = pJoint->shortConstSquared / (dotProd + pJoint->shortConstSquared) - 0.5f; 
    
    BS_VECTOR_SCALE(&delta, scaleFactor); 
    
    BS_VECTOR_SUB(pJoint->pAPos, &delta); 
    BS_VECTOR_ADD(pJoint->pBPos, &delta); 
  }
  else if(pJoint->longConstSquared != 0.0 && 
          dotProd > pJoint->longConstSquared)
  {
    scaleFactor = pJoint->longConstSquared / (dotProd + pJoint->longConstSquared) - 0.5f; 
    
    BS_VECTOR_SCALE(&delta, scaleFactor); 
    
    BS_VECTOR_SUB(pJoint->pAPos, &delta); 
    BS_VECTOR_ADD(pJoint->pBPos, &delta); 
  }
}
