#ifndef __BS_POINTMASS_H__
#define __BS_POINTMASS_H__

#include "bs_types.h"
#include "bs_vector.h"

typedef struct bs_pointmass_st
{
  bs_vector_t *pCurPos; 
  bs_vector_t *pPrevPos; 
  bs_vector_t *pForce; 
  bs_float_t mass;   
  bs_float_t friction; 
} bs_pointmass_t; 

bs_pointmass_t* 
bs_pointmass_create(bs_float_t xPos, 
                    bs_float_t yPos, 
                    bs_float_t mass);

void 
bs_pointmass_destroy(bs_pointmass_t *pPointmass); 

bs_float_t 
bs_pointmass_get_x_pos(bs_pointmass_t *pPointmass); 

bs_float_t 
bs_pointmass_get_y_pos(bs_pointmass_t *pPointmass); 

bs_vector_t* 
bs_pointmass_get_pos(bs_pointmass_t *pPointmass); 

bs_float_t 
bs_pointmass_get_x_prev_pos(bs_pointmass_t *pPointmass); 

bs_float_t 
bs_pointmass_get_y_prev_pos(bs_pointmass_t *pPointmass); 

bs_vector_t* 
bs_pointmass_get_prev_pos(bs_pointmass_t *pPointmass); 

void 
bs_pointmass_set_x_pos(bs_pointmass_t *pPointmass, 
                       bs_float_t x); 

void 
bs_pointmass_set_y_pos(bs_pointmass_t *pPointmass, 
                       bs_float_t y); 

void 
bs_pointmass_set_pos(bs_pointmass_t *pPointmass, 
                     bs_vector_t *pPos); 

void 
bs_pointmass_add_x_pos(bs_pointmass_t *pPointmass, 
                       bs_float_t x); 

void 
bs_pointmass_add_y_pos(bs_pointmass_t *pPointmass, 
                       bs_float_t y); 

void 
bs_pointmass_add_pos(bs_pointmass_t *pPointmass, 
                     bs_vector_t *pPos); 

void 
bs_pointmass_set_x_force(bs_pointmass_t *pPointmass, 
                         bs_float_t x); 

void 
bs_pointmass_set_y_force(bs_pointmass_t *pPointmass, 
                         bs_float_t y); 

void 
bs_pointmass_set_force(bs_pointmass_t *pPointmass, 
                       bs_vector_t *pForce); 

void 
bs_pointmass_add_x_force(bs_pointmass_t *pPointmass, 
                         bs_float_t x); 

void 
bs_pointmass_add_y_force(bs_pointmass_t *pPointmass, 
                         bs_float_t y); 

void 
bs_pointmass_add_force(bs_pointmass_t *pPointmass, 
                       bs_vector_t *pForce); 

bs_float_t
bs_pointmass_get_mass(bs_pointmass_t *pPointmass);

void 
bs_pointmass_set_mass(bs_pointmass_t *pPointmass, 
                      bs_float_t mass);

void 
bs_pointmass_set_friction(bs_pointmass_t *pPointmass, 
                          bs_float_t friction); 

void 
bs_pointmass_move(bs_pointmass_t *pPointmass,
                  bs_float_t dt); 

bs_float_t 
bs_pointmass_get_square_vel(bs_pointmass_t *pPointmass); 

#endif
